# Form implementation generated from reading ui file 'lifescale/gui/MainWindow.ui'
#
# Created by: PyQt6 UI code generator 6.4.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1277, 728)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget_Main = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget_Main.setTabPosition(QtWidgets.QTabWidget.TabPosition.East)
        self.tabWidget_Main.setObjectName("tabWidget_Main")
        self.tab_LSData = QtWidgets.QWidget()
        self.tab_LSData.setObjectName("tab_LSData")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tab_LSData)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.splitter = QtWidgets.QSplitter(self.tab_LSData)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.splitter.sizePolicy().hasHeightForWidth())
        self.splitter.setSizePolicy(sizePolicy)
        self.splitter.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.splitter.setChildrenCollapsible(False)
        self.splitter.setObjectName("splitter")
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout_2.setSizeConstraint(QtWidgets.QLayout.SizeConstraint.SetDefaultConstraint)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.listWidget_Samples = QtWidgets.QListWidget(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listWidget_Samples.sizePolicy().hasHeightForWidth())
        self.listWidget_Samples.setSizePolicy(sizePolicy)
        self.listWidget_Samples.setMinimumSize(QtCore.QSize(150, 0))
        self.listWidget_Samples.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.listWidget_Samples.setObjectName("listWidget_Samples")
        self.verticalLayout_2.addWidget(self.listWidget_Samples)
        self.groupBox_LSDataOptions = QtWidgets.QGroupBox(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_LSDataOptions.sizePolicy().hasHeightForWidth())
        self.groupBox_LSDataOptions.setSizePolicy(sizePolicy)
        self.groupBox_LSDataOptions.setMinimumSize(QtCore.QSize(150, 0))
        self.groupBox_LSDataOptions.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.groupBox_LSDataOptions.setObjectName("groupBox_LSDataOptions")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.groupBox_LSDataOptions)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.pushButton_resetMassLimits = QtWidgets.QPushButton(self.groupBox_LSDataOptions)
        self.pushButton_resetMassLimits.setObjectName("pushButton_resetMassLimits")
        self.verticalLayout_8.addWidget(self.pushButton_resetMassLimits)
        self.checkBox_MassesShowTable = QtWidgets.QCheckBox(self.groupBox_LSDataOptions)
        self.checkBox_MassesShowTable.setChecked(True)
        self.checkBox_MassesShowTable.setObjectName("checkBox_MassesShowTable")
        self.verticalLayout_8.addWidget(self.checkBox_MassesShowTable)
        self.verticalLayout_2.addWidget(self.groupBox_LSDataOptions)
        self.tabWidget_LSData = QtWidgets.QTabWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabWidget_LSData.sizePolicy().hasHeightForWidth())
        self.tabWidget_LSData.setSizePolicy(sizePolicy)
        self.tabWidget_LSData.setObjectName("tabWidget_LSData")
        self.tab_LSData_hist = QtWidgets.QWidget()
        self.tab_LSData_hist.setObjectName("tab_LSData_hist")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.tab_LSData_hist)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.GraphicsLayoutWidget_SampelHistogram = GraphicsLayoutWidget(self.tab_LSData_hist)
        self.GraphicsLayoutWidget_SampelHistogram.setObjectName("GraphicsLayoutWidget_SampelHistogram")
        self.verticalLayout_9.addWidget(self.GraphicsLayoutWidget_SampelHistogram)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.groupBoxLSDataSampleStatistics = QtWidgets.QGroupBox(self.tab_LSData_hist)
        self.groupBoxLSDataSampleStatistics.setObjectName("groupBoxLSDataSampleStatistics")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.groupBoxLSDataSampleStatistics)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label)
        self.label_SelectedMasse = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_SelectedMasse.setText("")
        self.label_SelectedMasse.setObjectName("label_SelectedMasse")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_SelectedMasse)
        self.label_3 = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_3)
        self.label_selectionLimits = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_selectionLimits.setText("")
        self.label_selectionLimits.setObjectName("label_selectionLimits")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_selectionLimits)
        self.label_5 = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_5)
        self.label_mean = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_mean.setText("")
        self.label_mean.setObjectName("label_mean")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_mean)
        self.label_7 = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_7.setObjectName("label_7")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_7)
        self.label_median = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_median.setText("")
        self.label_median.setObjectName("label_median")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_median)
        self.label_9 = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_9.setObjectName("label_9")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_9)
        self.label_std = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_std.setText("")
        self.label_std.setObjectName("label_std")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_std)
        self.label_11 = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_11.setObjectName("label_11")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_11)
        self.label_iqr = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_iqr.setText("")
        self.label_iqr.setObjectName("label_iqr")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_iqr)
        self.label_13 = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_13.setObjectName("label_13")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_13)
        self.label_rcv = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_rcv.setText("")
        self.label_rcv.setObjectName("label_rcv")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_rcv)
        self.horizontalLayout.addLayout(self.formLayout)
        self.horizontalLayout_2.addWidget(self.groupBoxLSDataSampleStatistics)
        self.groupBox_histOptions = QtWidgets.QGroupBox(self.tab_LSData_hist)
        self.groupBox_histOptions.setObjectName("groupBox_histOptions")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_histOptions)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setObjectName("formLayout_2")
        self.label_2 = QtWidgets.QLabel(self.groupBox_histOptions)
        self.label_2.setObjectName("label_2")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_2)
        self.comboBox_histMethod = QtWidgets.QComboBox(self.groupBox_histOptions)
        self.comboBox_histMethod.setObjectName("comboBox_histMethod")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.comboBox_histMethod)
        self.label_numberOfBins = QtWidgets.QLabel(self.groupBox_histOptions)
        self.label_numberOfBins.setObjectName("label_numberOfBins")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_numberOfBins)
        self.spinBox_histBins = QtWidgets.QSpinBox(self.groupBox_histOptions)
        self.spinBox_histBins.setMaximum(1000)
        self.spinBox_histBins.setStepType(QtWidgets.QAbstractSpinBox.StepType.AdaptiveDecimalStepType)
        self.spinBox_histBins.setProperty("value", 100)
        self.spinBox_histBins.setObjectName("spinBox_histBins")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.spinBox_histBins)
        self.verticalLayout_3.addLayout(self.formLayout_2)
        self.label_histBinMethodDescription = QtWidgets.QLabel(self.groupBox_histOptions)
        self.label_histBinMethodDescription.setText("")
        self.label_histBinMethodDescription.setWordWrap(True)
        self.label_histBinMethodDescription.setObjectName("label_histBinMethodDescription")
        self.verticalLayout_3.addWidget(self.label_histBinMethodDescription)
        self.checkBox_plotMedian = QtWidgets.QCheckBox(self.groupBox_histOptions)
        self.checkBox_plotMedian.setChecked(False)
        self.checkBox_plotMedian.setObjectName("checkBox_plotMedian")
        self.verticalLayout_3.addWidget(self.checkBox_plotMedian)
        self.checkBox_plotMean = QtWidgets.QCheckBox(self.groupBox_histOptions)
        self.checkBox_plotMean.setChecked(False)
        self.checkBox_plotMean.setObjectName("checkBox_plotMean")
        self.verticalLayout_3.addWidget(self.checkBox_plotMean)
        self.horizontalLayout_2.addWidget(self.groupBox_histOptions)
        self.verticalLayout_9.addLayout(self.horizontalLayout_2)
        self.tabWidget_LSData.addTab(self.tab_LSData_hist, "")
        self.tab_Masses = QtWidgets.QWidget()
        self.tab_Masses.setObjectName("tab_Masses")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.tab_Masses)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.tableView_Masses = QtWidgets.QTableView(self.tab_Masses)
        self.tableView_Masses.setEnabled(True)
        self.tableView_Masses.setLineWidth(9)
        self.tableView_Masses.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.SizeAdjustPolicy.AdjustToContents)
        self.tableView_Masses.setObjectName("tableView_Masses")
        self.verticalLayout_5.addWidget(self.tableView_Masses)
        self.tabWidget_LSData.addTab(self.tab_Masses, "")
        self.verticalLayout_4.addWidget(self.splitter)
        self.tabWidget_Main.addTab(self.tab_LSData, "")
        self.tab_SampleSummary = QtWidgets.QWidget()
        self.tab_SampleSummary.setObjectName("tab_SampleSummary")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.tab_SampleSummary)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.tableView_SampleSummary = QtWidgets.QTableView(self.tab_SampleSummary)
        self.tableView_SampleSummary.setObjectName("tableView_SampleSummary")
        self.verticalLayout_7.addWidget(self.tableView_SampleSummary)
        self.tabWidget_Main.addTab(self.tab_SampleSummary, "")
        self.tab_PanelMetadata = QtWidgets.QWidget()
        self.tab_PanelMetadata.setObjectName("tab_PanelMetadata")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.tab_PanelMetadata)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.tableView_PanelMetadata = QtWidgets.QTableView(self.tab_PanelMetadata)
        self.tableView_PanelMetadata.setObjectName("tableView_PanelMetadata")
        self.verticalLayout_6.addWidget(self.tableView_PanelMetadata)
        self.tabWidget_Main.addTab(self.tab_PanelMetadata, "")
        self.verticalLayout.addWidget(self.tabWidget_Main)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1277, 22))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuExport = QtWidgets.QMenu(self.menuFile)
        self.menuExport.setObjectName("menuExport")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionLoad_xlsm_file = QtGui.QAction(MainWindow)
        self.actionLoad_xlsm_file.setObjectName("actionLoad_xlsm_file")
        self.action_About = QtGui.QAction(MainWindow)
        self.action_About.setObjectName("action_About")
        self.action_csvFiles = QtGui.QAction(MainWindow)
        self.action_csvFiles.setObjectName("action_csvFiles")
        self.action_Exit = QtGui.QAction(MainWindow)
        self.action_Exit.setObjectName("action_Exit")
        self.menuExport.addAction(self.action_csvFiles)
        self.menuFile.addAction(self.actionLoad_xlsm_file)
        self.menuFile.addAction(self.menuExport.menuAction())
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.action_Exit)
        self.menuHelp.addAction(self.action_About)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget_Main.setCurrentIndex(0)
        self.tabWidget_LSData.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "LifeScale utils"))
        self.groupBox_LSDataOptions.setTitle(_translate("MainWindow", "Options"))
        self.pushButton_resetMassLimits.setText(_translate("MainWindow", "Reset mass limits"))
        self.checkBox_MassesShowTable.setText(_translate("MainWindow", "Show table"))
        self.groupBoxLSDataSampleStatistics.setTitle(_translate("MainWindow", "Sample statistics (current selection)"))
        self.label.setText(_translate("MainWindow", "Selected masses"))
        self.label_3.setText(_translate("MainWindow", "Selection limits"))
        self.label_5.setText(_translate("MainWindow", "Mean"))
        self.label_7.setText(_translate("MainWindow", "Median"))
        self.label_9.setText(_translate("MainWindow", "Std"))
        self.label_11.setText(_translate("MainWindow", "IQR"))
        self.label_13.setText(_translate("MainWindow", "rCV"))
        self.groupBox_histOptions.setTitle(_translate("MainWindow", "Histogram options"))
        self.label_2.setText(_translate("MainWindow", "Bin determination method"))
        self.label_numberOfBins.setText(_translate("MainWindow", "Number of bins"))
        self.checkBox_plotMedian.setText(_translate("MainWindow", "Plot median"))
        self.checkBox_plotMean.setText(_translate("MainWindow", "Plot mean"))
        self.tabWidget_LSData.setTabText(self.tabWidget_LSData.indexOf(self.tab_LSData_hist), _translate("MainWindow", "Sample histogram and statistics"))
        self.tabWidget_LSData.setTabText(self.tabWidget_LSData.indexOf(self.tab_Masses), _translate("MainWindow", "Masses per Sample"))
        self.tabWidget_Main.setTabText(self.tabWidget_Main.indexOf(self.tab_LSData), _translate("MainWindow", "Masses"))
        self.tabWidget_Main.setTabText(self.tabWidget_Main.indexOf(self.tab_SampleSummary), _translate("MainWindow", "Sample Summary"))
        self.tabWidget_Main.setTabText(self.tabWidget_Main.indexOf(self.tab_PanelMetadata), _translate("MainWindow", "Panel Metadata"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.menuExport.setTitle(_translate("MainWindow", "Export"))
        self.menuHelp.setTitle(_translate("MainWindow", "Help"))
        self.actionLoad_xlsm_file.setText(_translate("MainWindow", "Load xlsm file"))
        self.actionLoad_xlsm_file.setToolTip(_translate("MainWindow", "Load xlsm file created by LifeScale"))
        self.action_About.setText(_translate("MainWindow", "About"))
        self.action_csvFiles.setText(_translate("MainWindow", "CSV files"))
        self.action_Exit.setText(_translate("MainWindow", "Exit"))
from pyqtgraph import GraphicsLayoutWidget
